"""
Common options for serving.
"""
import sys
from typing import Optional

import click

from openapi_client import ResponseModelServiceInfo
from vessl import vessl_api
from vessl.util.exception import VesslApiException


def serving_name_callback(
    ctx: click.Context, param: click.Parameter, serving_name: Optional[str]
) -> ResponseModelServiceInfo:
    if vessl_api.organization is None:
        vessl_api.set_organization()

    try:
        serving: ResponseModelServiceInfo = vessl_api.model_service_read_api(
            serving_name, vessl_api.organization.name
        )
    except VesslApiException as e:
        print(f"Invalid serving {serving_name}: {e.message}")
        sys.exit(1)

    return serving


serving_name_option = click.option(
    "--serving",
    type=click.STRING,
    required=True,
    callback=serving_name_callback,
    help="Name of serving.",
)

update_gateway_option = click.option(
    "--update-gateway/--no-update-gateway",
    "-g/-G",
    is_flag=True,
    default=False,
    help="Whether to update gateway so that it points to this revision.",
)

enable_gateway_if_off_option = click.option(
    "--enable-gateway-if-off/--no-enable-gateway-if-off",
    "-e/-E",
    is_flag=True,
    default=False,
    help="When updating gateway, whether to enable the gateway if it is currently off.",
)

update_gateway_weight_option = click.option(
    "--update-gateway-weight",
    type=click.INT,
    required=False,
    help=(
        "When updating gateway, the amount of traffic that should be "
        "directed to this revision (in percentage)."
    ),
)

update_gateway_port_option = click.option(
    "--update-gateway-port",
    type=click.INT,
    required=False,
    help=(
        "When updating gateway, the port to receive the traffic; "
        "this port must be defined in serving spec."
    ),
)



from test.conftest import TEST_CLUSTER_NAME, USE_MOCK

import pytest

import vessl


@pytest.mark.skipif(USE_MOCK, reason="Does not run if mocking is used.")
class TestKernelCluster:
    def test_read_cluster(self):
        vessl.read_cluster(TEST_CLUSTER_NAME)

    def test_list_clusters(self):
        vessl.list_clusters()

    def test_delete_cluster(self):
        # Do not test delete cluster because only one cluster exists.
        pass

    def test_rename_cluster(self):
        # Only applies to custom clusters
        pass

    def test_list_cluster_nodes(self):
        # Only applies to custom clusters
        pass



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class V1MountForm(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cifs': 'object',
        'container_storage': 'object',
        'dataset': 'V1DatasetRefForm',
        'google_disk': 'V1GoogleDiskRefForm',
        'hostpath': 'V1HostPathRefForm',
        'nfs': 'V1NFSRefForm',
        'options': 'V1MountOptions',
        'ref_type': 'str'
    }

    attribute_map = {
        'cifs': 'cifs',
        'container_storage': 'container_storage',
        'dataset': 'dataset',
        'google_disk': 'google_disk',
        'hostpath': 'hostpath',
        'nfs': 'nfs',
        'options': 'options',
        'ref_type': 'ref_type'
    }

    def __init__(self, cifs=None, container_storage=None, dataset=None, google_disk=None, hostpath=None, nfs=None, options=None, ref_type=None, local_vars_configuration=None):  # noqa: E501
        """V1MountForm - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cifs = None
        self._container_storage = None
        self._dataset = None
        self._google_disk = None
        self._hostpath = None
        self._nfs = None
        self._options = None
        self._ref_type = None
        self.discriminator = None

        if cifs is not None:
            self.cifs = cifs
        if container_storage is not None:
            self.container_storage = container_storage
        if dataset is not None:
            self.dataset = dataset
        if google_disk is not None:
            self.google_disk = google_disk
        if hostpath is not None:
            self.hostpath = hostpath
        if nfs is not None:
            self.nfs = nfs
        if options is not None:
            self.options = options
        if ref_type is not None:
            self.ref_type = ref_type

    @property
    def cifs(self):
        """Gets the cifs of this V1MountForm.  # noqa: E501


        :return: The cifs of this V1MountForm.  # noqa: E501
        :rtype: object
        """
        return self._cifs

    @cifs.setter
    def cifs(self, cifs):
        """Sets the cifs of this V1MountForm.


        :param cifs: The cifs of this V1MountForm.  # noqa: E501
        :type cifs: object
        """

        self._cifs = cifs

    @property
    def container_storage(self):
        """Gets the container_storage of this V1MountForm.  # noqa: E501


        :return: The container_storage of this V1MountForm.  # noqa: E501
        :rtype: object
        """
        return self._container_storage

    @container_storage.setter
    def container_storage(self, container_storage):
        """Sets the container_storage of this V1MountForm.


        :param container_storage: The container_storage of this V1MountForm.  # noqa: E501
        :type container_storage: object
        """

        self._container_storage = container_storage

    @property
    def dataset(self):
        """Gets the dataset of this V1MountForm.  # noqa: E501


        :return: The dataset of this V1MountForm.  # noqa: E501
        :rtype: V1DatasetRefForm
        """
        return self._dataset

    @dataset.setter
    def dataset(self, dataset):
        """Sets the dataset of this V1MountForm.


        :param dataset: The dataset of this V1MountForm.  # noqa: E501
        :type dataset: V1DatasetRefForm
        """

        self._dataset = dataset

    @property
    def google_disk(self):
        """Gets the google_disk of this V1MountForm.  # noqa: E501


        :return: The google_disk of this V1MountForm.  # noqa: E501
        :rtype: V1GoogleDiskRefForm
        """
        return self._google_disk

    @google_disk.setter
    def google_disk(self, google_disk):
        """Sets the google_disk of this V1MountForm.


        :param google_disk: The google_disk of this V1MountForm.  # noqa: E501
        :type google_disk: V1GoogleDiskRefForm
        """

        self._google_disk = google_disk

    @property
    def hostpath(self):
        """Gets the hostpath of this V1MountForm.  # noqa: E501


        :return: The hostpath of this V1MountForm.  # noqa: E501
        :rtype: V1HostPathRefForm
        """
        return self._hostpath

    @hostpath.setter
    def hostpath(self, hostpath):
        """Sets the hostpath of this V1MountForm.


        :param hostpath: The hostpath of this V1MountForm.  # noqa: E501
        :type hostpath: V1HostPathRefForm
        """

        self._hostpath = hostpath

    @property
    def nfs(self):
        """Gets the nfs of this V1MountForm.  # noqa: E501


        :return: The nfs of this V1MountForm.  # noqa: E501
        :rtype: V1NFSRefForm
        """
        return self._nfs

    @nfs.setter
    def nfs(self, nfs):
        """Sets the nfs of this V1MountForm.


        :param nfs: The nfs of this V1MountForm.  # noqa: E501
        :type nfs: V1NFSRefForm
        """

        self._nfs = nfs

    @property
    def options(self):
        """Gets the options of this V1MountForm.  # noqa: E501


        :return: The options of this V1MountForm.  # noqa: E501
        :rtype: V1MountOptions
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this V1MountForm.


        :param options: The options of this V1MountForm.  # noqa: E501
        :type options: V1MountOptions
        """

        self._options = options

    @property
    def ref_type(self):
        """Gets the ref_type of this V1MountForm.  # noqa: E501


        :return: The ref_type of this V1MountForm.  # noqa: E501
        :rtype: str
        """
        return self._ref_type

    @ref_type.setter
    def ref_type(self, ref_type):
        """Sets the ref_type of this V1MountForm.


        :param ref_type: The ref_type of this V1MountForm.  # noqa: E501
        :type ref_type: str
        """
        allowed_values = ["dataset", "hostpath", "nfs", "cifs", "google_disk", "container_storage"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and ref_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `ref_type` ({0}), must be one of {1}"  # noqa: E501
                .format(ref_type, allowed_values)
            )

        self._ref_type = ref_type

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1MountForm):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1MountForm):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class V1ResourcesForm(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster': 'str',
        'cluster_id': 'int',
        'node_id': 'list[int]',
        'node_names': 'list[str]',
        'preset': 'str',
        'preset_id': 'int',
        'requests': 'V1ResourceRequest',
        'spot': 'str'
    }

    attribute_map = {
        'cluster': 'cluster',
        'cluster_id': 'cluster_id',
        'node_id': 'node_id',
        'node_names': 'node_names',
        'preset': 'preset',
        'preset_id': 'preset_id',
        'requests': 'requests',
        'spot': 'spot'
    }

    def __init__(self, cluster=None, cluster_id=None, node_id=None, node_names=None, preset=None, preset_id=None, requests=None, spot=None, local_vars_configuration=None):  # noqa: E501
        """V1ResourcesForm - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cluster = None
        self._cluster_id = None
        self._node_id = None
        self._node_names = None
        self._preset = None
        self._preset_id = None
        self._requests = None
        self._spot = None
        self.discriminator = None

        self.cluster = cluster
        self.cluster_id = cluster_id
        if node_id is not None:
            self.node_id = node_id
        if node_names is not None:
            self.node_names = node_names
        self.preset = preset
        self.preset_id = preset_id
        if requests is not None:
            self.requests = requests
        self.spot = spot

    @property
    def cluster(self):
        """Gets the cluster of this V1ResourcesForm.  # noqa: E501


        :return: The cluster of this V1ResourcesForm.  # noqa: E501
        :rtype: str
        """
        return self._cluster

    @cluster.setter
    def cluster(self, cluster):
        """Sets the cluster of this V1ResourcesForm.


        :param cluster: The cluster of this V1ResourcesForm.  # noqa: E501
        :type cluster: str
        """

        self._cluster = cluster

    @property
    def cluster_id(self):
        """Gets the cluster_id of this V1ResourcesForm.  # noqa: E501


        :return: The cluster_id of this V1ResourcesForm.  # noqa: E501
        :rtype: int
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this V1ResourcesForm.


        :param cluster_id: The cluster_id of this V1ResourcesForm.  # noqa: E501
        :type cluster_id: int
        """

        self._cluster_id = cluster_id

    @property
    def node_id(self):
        """Gets the node_id of this V1ResourcesForm.  # noqa: E501


        :return: The node_id of this V1ResourcesForm.  # noqa: E501
        :rtype: list[int]
        """
        return self._node_id

    @node_id.setter
    def node_id(self, node_id):
        """Sets the node_id of this V1ResourcesForm.


        :param node_id: The node_id of this V1ResourcesForm.  # noqa: E501
        :type node_id: list[int]
        """

        self._node_id = node_id

    @property
    def node_names(self):
        """Gets the node_names of this V1ResourcesForm.  # noqa: E501


        :return: The node_names of this V1ResourcesForm.  # noqa: E501
        :rtype: list[str]
        """
        return self._node_names

    @node_names.setter
    def node_names(self, node_names):
        """Sets the node_names of this V1ResourcesForm.


        :param node_names: The node_names of this V1ResourcesForm.  # noqa: E501
        :type node_names: list[str]
        """

        self._node_names = node_names

    @property
    def preset(self):
        """Gets the preset of this V1ResourcesForm.  # noqa: E501


        :return: The preset of this V1ResourcesForm.  # noqa: E501
        :rtype: str
        """
        return self._preset

    @preset.setter
    def preset(self, preset):
        """Sets the preset of this V1ResourcesForm.


        :param preset: The preset of this V1ResourcesForm.  # noqa: E501
        :type preset: str
        """

        self._preset = preset

    @property
    def preset_id(self):
        """Gets the preset_id of this V1ResourcesForm.  # noqa: E501


        :return: The preset_id of this V1ResourcesForm.  # noqa: E501
        :rtype: int
        """
        return self._preset_id

    @preset_id.setter
    def preset_id(self, preset_id):
        """Sets the preset_id of this V1ResourcesForm.


        :param preset_id: The preset_id of this V1ResourcesForm.  # noqa: E501
        :type preset_id: int
        """

        self._preset_id = preset_id

    @property
    def requests(self):
        """Gets the requests of this V1ResourcesForm.  # noqa: E501


        :return: The requests of this V1ResourcesForm.  # noqa: E501
        :rtype: V1ResourceRequest
        """
        return self._requests

    @requests.setter
    def requests(self, requests):
        """Sets the requests of this V1ResourcesForm.


        :param requests: The requests of this V1ResourcesForm.  # noqa: E501
        :type requests: V1ResourceRequest
        """

        self._requests = requests

    @property
    def spot(self):
        """Gets the spot of this V1ResourcesForm.  # noqa: E501

        oneof:<empty string>,SpotOnly,SpotFirst,OnDemandOnly  # noqa: E501

        :return: The spot of this V1ResourcesForm.  # noqa: E501
        :rtype: str
        """
        return self._spot

    @spot.setter
    def spot(self, spot):
        """Sets the spot of this V1ResourcesForm.

        oneof:<empty string>,SpotOnly,SpotFirst,OnDemandOnly  # noqa: E501

        :param spot: The spot of this V1ResourcesForm.  # noqa: E501
        :type spot: str
        """

        self._spot = spot

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, V1ResourcesForm):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, V1ResourcesForm):
            return True

        return self.to_dict() != other.to_dict()



import os
from pathlib import Path
from typing import Any, Dict, List, Optional

import click

from vessl import list_ssh_keys
from vessl.cli._base import VesslGroup, vessl_argument, vessl_option
from vessl.cli._util import (
    Endpoint,
    generic_prompter,
    print_data,
    print_info,
    print_logs,
    print_success,
    print_table,
    prompt_choices,
    truncate_datetime,
)
from vessl.cli.experiment import (
    cpu_limit_option,
    dataset_option,
    gpu_limit_option,
    gpu_type_option,
    image_url_option,
    memory_limit_option,
    processor_type_option,
    root_volume_size_option,
    upload_local_file_option,
    use_vesslignore_option,
)
from vessl.cli.kernel_cluster import cluster_option
from vessl.cli.kernel_resource_spec import resource_option
from vessl.cli.organization import organization_name_option
from vessl.util.common import safe_cast
from vessl.util.constant import SSH_CONFIG_PATH
from vessl.workspace import (
    backup_workspace,
    connect_workspace_ssh,
    create_workspace,
    list_workspace_logs,
    list_workspaces,
    read_workspace,
    restore_workspace,
    start_workspace,
    stop_workspace,
    terminate_workspace,
    update_vscode_remote_ssh,
)


class WorkspacePortType(click.ParamType):
    name = "Port type"

    def convert(self, raw_value: Any, param, ctx) -> Any:
        tokens = raw_value.split()

        if len(tokens) < 3:
            raise click.BadOptionUsage(
                option_name="port",
                message=f"Invalid value for [PORT]: '{raw_value}' must be of form [expose_type] [port] [name].",
            )

        port_num = safe_cast(tokens[1], to_type=int)
        if port_num is None:
            raise click.BadOptionUsage(
                option_name="port",
                message=f"Invalid value for [PORT]: 'port' must be a number.",
            )

        port = {"expose_type": tokens[0], "port": port_num, "name": tokens[2]}
        return port


def ssh_private_key_path_callback(
    ctx: click.Context, param: click.Parameter, key_path: str
) -> Optional[str]:
    if key_path:
        return key_path

    ssh_keys = list_ssh_keys()
    if len(ssh_keys) == 0:
        raise click.BadParameter(
            "At least one ssh public key should be added.\n" "Please run `vessl ssh-key add`."
        )

    for ssh_key in ssh_keys:
        key_path = os.path.join(Path.home(), ".ssh", ssh_key.filename.rstrip(".pub"))
        if os.path.exists(key_path):
            return key_path

    for key_path in (
        os.path.join(Path.home(), ".ssh", "id_rsa"),
        os.path.join(Path.home(), ".ssh", "id_ed25519"),
    ):
        if os.path.exists(key_path):
            return key_path

    return None


def workspace_id_prompter(ctx: click.Context, param: click.Parameter, value: int) -> int:
    workspaces = list_workspaces()
    if len(workspaces) == 0:
        raise click.UsageError("No valid workspaces.")
    return prompt_choices("Workspace", [(x.name, x.id) for x in workspaces])


def start_workspace_id_prompter(ctx: click.Context, param: click.Parameter, value: int) -> int:
    workspaces = list_workspaces(statuses=["stopped"])
    if len(workspaces) == 0:
        raise click.UsageError("No valid workspaces.")
    return prompt_choices("Workspace", [(x.name, x.id) for x in workspaces])


def stop_workspace_id_prompter(ctx: click.Context, param: click.Parameter, value: int) -> int:
    workspaces = list_workspaces(statuses=["queued", "pending", "initializing", "running"])
    if len(workspaces) == 0:
        raise click.UsageError("No valid workspaces.")
    return prompt_choices("Workspace", [(x.name, x.id) for x in workspaces])


@click.command(name="workspace", cls=VesslGroup)
def cli():
    pass


@cli.vessl_command()
@organization_name_option
def list():
    workspaces = list_workspaces()
    print_table(
        workspaces,
        ["ID", "Name", "Status", "Created", "Creator", "Cluster", "Resource"],
        lambda x: [
            x.id,
            x.name,
            x.status,
            truncate_datetime(x.created_dt),
            x.created_by.username,
            x.kernel_cluster.name,
            x.kernel_resource_spec.name,
        ],
    )


@cli.vessl_command()
@vessl_argument(
    "id",
    type=click.INT,
    required=True,
    prompter=workspace_id_prompter,
)
@organization_name_option
def read(id: int):
    workspace = read_workspace(workspace_id=id)
    print_data(
        {
            "ID": workspace.id,
            "Name": workspace.name,
            "Status": workspace.status,
            "Created": truncate_datetime(workspace.created_dt),
            "Creator": workspace.created_by.username,
            "Cluster": workspace.kernel_cluster.name,
            "Image": workspace.kernel_image.name,
            "Resource": workspace.kernel_resource_spec.name,
        }
    )
    print_info(
        f"For more info: {Endpoint.workspace.format(workspace.organization.name, workspace.id)}"
    )


@cli.vessl_command()
@vessl_argument("name", type=click.STRING, prompter=generic_prompter("Name"))
@cluster_option
@vessl_option(
    "--node",
    type=click.STRING,
    multiple=True,
    help="Cluster nodes. Defaults to all nodes in cluster.",
)
@resource_option
@processor_type_option
@cpu_limit_option
@memory_limit_option
@gpu_type_option
@gpu_limit_option
@image_url_option
@vessl_option(
    "--max-hours",
    type=click.INT,
    default=24,
    help="Maximum number of hours to run workspace. Defaults to 24.",
)
@dataset_option
@upload_local_file_option
@use_vesslignore_option
@root_volume_size_option
@vessl_option(
    "-p",
    "--port",
    type=WorkspacePortType(),
    multiple=True,
    help="Format: [expose_type] [port] [name], ex. `-p 'tcp 22 ssh'`. Jupyter and SSH ports exist by default.",
)
@vessl_option("--init-script", type=click.STRING, help="Custom init script")
@organization_name_option
def create(
    name: str,
    cluster: str,
    node: List[str],
    resource: str,
    processor_type: str,
    cpu_limit: float,
    memory_limit: str,
    gpu_type: str,
    gpu_limit: int,
    image_url: str,
    max_hours: int,
    dataset: List[str],
    upload_local_file: List[str],
    use_vesslignore: bool,
    root_volume_size: str,
    port: List[Dict[str, Any]],
    init_script: str,
):
    workspace = create_workspace(
        name=name,
        cluster_name=cluster,
        cluster_node_names=node,
        kernel_resource_spec_name=resource,
        processor_type=processor_type,
        cpu_limit=cpu_limit,
        memory_limit=memory_limit,
        gpu_type=gpu_type,
        gpu_limit=gpu_limit,
        kernel_image_url=image_url,
        max_hours=max_hours,
        dataset_mounts=dataset,
        local_files=upload_local_file,
        use_vesslignore=use_vesslignore,
        root_volume_size=root_volume_size,
        ports=port,
        init_script=init_script,
    )
    print_success(
        f"Created '{workspace.name}'.\n"
        f"For more info: {Endpoint.workspace.format(workspace.organization.name, workspace.id)}"
    )


@cli.vessl_command()
@vessl_argument(
    "id",
    type=click.INT,
    required=True,
    prompter=workspace_id_prompter,
)
@click.option(
    "--tail",
    type=click.INT,
    default=200,
    help="Number of lines to display (from the end).",
)
@organization_name_option
def logs(id: int, tail: int):
    logs = list_workspace_logs(workspace_id=id, tail=tail)
    print_logs(logs)


@cli.vessl_command()
@vessl_argument(
    "id",
    type=click.INT,
    required=True,
    prompter=start_workspace_id_prompter,
)
@organization_name_option
def start(id: int):
    workspace = start_workspace(workspace_id=id)
    print_success(
        f"Started '{workspace.name}'.\n"
        f"For more info: {Endpoint.workspace.format(workspace.organization.name, workspace.id)}"
    )


@cli.vessl_command()
@vessl_argument(
    "id",
    type=click.INT,
    required=True,
    prompter=stop_workspace_id_prompter,
)
@organization_name_option
def stop(id: int):
    workspace = stop_workspace(workspace_id=id)
    print_success(
        f"Stopped '{workspace.name}'.\n"
        f"For more info: {Endpoint.workspace.format(workspace.organization.name, workspace.id)}"
    )


@cli.vessl_command()
@vessl_argument(
    "id",
    type=click.INT,
    required=True,
    prompter=workspace_id_prompter,
)
@organization_name_option
def terminate(id: int):
    workspace = terminate_workspace(workspace_id=id)
    print_success(
        f"Terminated '{workspace.name}'.\n"
        f"For more info: {Endpoint.workspace.format(workspace.organization.name, workspace.id)}"
    )


@cli.vessl_command()
@vessl_option(
    "-p",
    "--key-path",
    type=click.Path(exists=True),
    callback=ssh_private_key_path_callback,
    help="Path to SSH private key.",
)
@organization_name_option
def ssh(key_path: str):
    connect_workspace_ssh(private_key_path=key_path)


@cli.vessl_command()
@vessl_option(
    "-p",
    "--key-path",
    type=click.STRING,
    callback=ssh_private_key_path_callback,
    help="SSH private key path.",
)
@organization_name_option
def vscode(key_path: str):
    update_vscode_remote_ssh(private_key_path=key_path)
    print_success(f"Updated '{SSH_CONFIG_PATH}'.")


@cli.vessl_command()
@organization_name_option
def backup():
    backup_workspace()


@cli.vessl_command()
@organization_name_option
def restore():
    restore_workspace()



import os
import pathlib
import subprocess
import tempfile
from typing import List

from openapi_client import ResponseCodeRefsV2, ResponseGitHubCodeRef
from vessl import logger
from vessl.util.downloader import Downloader


def clone_codes(code_refs: List[ResponseGitHubCodeRef]):
    for code_ref in code_refs:
        if code_ref.git_provider == "github":
            prefix = "x-access-token"
            git_provider_domain = "github.com"
        elif code_ref.git_provider == "gitlab":
            prefix = "oauth2"
            git_provider_domain = "gitlab.com"
        else:
            prefix = "x-token-auth"
            git_provider_domain = "bitbucket.org"

        if code_ref.git_provider_custom_domain is not None:
            git_provider_domain = code_ref.git_provider_custom_domain

        if code_ref.token is None or code_ref.token == "":
            git_url = f"https://{git_provider_domain}/{code_ref.git_owner}/{code_ref.git_repo}.git"
        else:
            git_url = f"https://{prefix}:{code_ref.token}@{git_provider_domain}/{code_ref.git_owner}/{code_ref.git_repo}.git"
        if code_ref.mount_path:
            dirname = code_ref.mount_path
        else:
            dirname = code_ref.git_repo

        try:
            subprocess.run(["git", "clone", git_url, dirname])
        except subprocess.CalledProcessError:
            dirname = f"vessl-{code_ref.git_repo}"
            logger.info(f"Falling back to '{dirname}'...")
            subprocess.run(["git", "clone", git_url, dirname])

        if code_ref.git_ref:
            subprocess.run(["/bin/sh", "-c", f"cd {dirname}; git reset --hard {code_ref.git_ref}"])

        if code_ref.git_diff_file:
            diff_file_path = f"/tmp/{code_ref.git_repo}.diff"
            Downloader.download(
                code_ref.git_diff_file.path, diff_file_path, code_ref.git_diff_file, quiet=False
            )
            subprocess.run(["/bin/sh", "-c", f"cd {dirname}; git apply {diff_file_path}"])


def clone_codes_v2(code_refs: List[ResponseCodeRefsV2]):
    for code_ref in code_refs:
        mount_path = pathlib.Path(code_ref.mount_path)
        fallback = False
        if mount_path.exists():
            if mount_path.is_file():
                print(f"path {code_ref.mount_path} is a file.")
                fallback = True
            elif any(mount_path.iterdir()):
                print(f"path {code_ref.mount_path} is not empty.")
                fallback = True
        if fallback:
            print(f"Warning: cannot clone into {mount_path}")
            dirname = mount_path.parent
            subdir = f"vessl-{mount_path.name}"
            mount_path = dirname / subdir
            print(f"Alternatively trying to clone into f{dirname}/{subdir}...")
            print(f"This might affect the automated code execution")
        if code_ref.protocol == "http":
            url = code_ref.ref_http.url
            subprocess.run(["git", "clone", url, str(mount_path)]).check_returncode()
        elif code_ref.protocol == "ssh":
            if code_ref.ref_ssh.private_key:
                with tempfile.TemporaryFile() as key:
                    key.write(code_ref.ref_ssh.private_key)
                    os.chmod(key.name, 0o400)
                    subprocess.run(
                        ["git", "clone", code_ref.ref_ssh.host, str(mount_path)],
                        env={
                            "GIT_SSH_COMMAND": f"ssh -i {key.name}",
                        },
                    ).check_returncode()
            else:
                subprocess.run(
                    ["git", "clone", code_ref.ref_ssh.host, str(mount_path)]
                ).check_returncode()

        if code_ref.git_ref:
            subprocess.run(
                ["git", "checkout", code_ref.git_ref], cwd=str(mount_path)
            ).check_returncode()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponsePlotFile(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'caption': 'str',
        'dest_path': 'str',
        'file': 'StorageFile',
        'file_type': 'str',
        'step': 'int'
    }

    attribute_map = {
        'caption': 'caption',
        'dest_path': 'dest_path',
        'file': 'file',
        'file_type': 'file_type',
        'step': 'step'
    }

    def __init__(self, caption=None, dest_path=None, file=None, file_type=None, step=None, local_vars_configuration=None):  # noqa: E501
        """ResponsePlotFile - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._caption = None
        self._dest_path = None
        self._file = None
        self._file_type = None
        self._step = None
        self.discriminator = None

        if caption is not None:
            self.caption = caption
        if dest_path is not None:
            self.dest_path = dest_path
        if file is not None:
            self.file = file
        if file_type is not None:
            self.file_type = file_type
        self.step = step

    @property
    def caption(self):
        """Gets the caption of this ResponsePlotFile.  # noqa: E501


        :return: The caption of this ResponsePlotFile.  # noqa: E501
        :rtype: str
        """
        return self._caption

    @caption.setter
    def caption(self, caption):
        """Sets the caption of this ResponsePlotFile.


        :param caption: The caption of this ResponsePlotFile.  # noqa: E501
        :type caption: str
        """

        self._caption = caption

    @property
    def dest_path(self):
        """Gets the dest_path of this ResponsePlotFile.  # noqa: E501


        :return: The dest_path of this ResponsePlotFile.  # noqa: E501
        :rtype: str
        """
        return self._dest_path

    @dest_path.setter
    def dest_path(self, dest_path):
        """Sets the dest_path of this ResponsePlotFile.


        :param dest_path: The dest_path of this ResponsePlotFile.  # noqa: E501
        :type dest_path: str
        """

        self._dest_path = dest_path

    @property
    def file(self):
        """Gets the file of this ResponsePlotFile.  # noqa: E501


        :return: The file of this ResponsePlotFile.  # noqa: E501
        :rtype: StorageFile
        """
        return self._file

    @file.setter
    def file(self, file):
        """Sets the file of this ResponsePlotFile.


        :param file: The file of this ResponsePlotFile.  # noqa: E501
        :type file: StorageFile
        """

        self._file = file

    @property
    def file_type(self):
        """Gets the file_type of this ResponsePlotFile.  # noqa: E501


        :return: The file_type of this ResponsePlotFile.  # noqa: E501
        :rtype: str
        """
        return self._file_type

    @file_type.setter
    def file_type(self, file_type):
        """Sets the file_type of this ResponsePlotFile.


        :param file_type: The file_type of this ResponsePlotFile.  # noqa: E501
        :type file_type: str
        """
        allowed_values = ["image", "audio"]  # noqa: E501
        if self.local_vars_configuration.client_side_validation and file_type not in allowed_values:  # noqa: E501
            raise ValueError(
                "Invalid value for `file_type` ({0}), must be one of {1}"  # noqa: E501
                .format(file_type, allowed_values)
            )

        self._file_type = file_type

    @property
    def step(self):
        """Gets the step of this ResponsePlotFile.  # noqa: E501


        :return: The step of this ResponsePlotFile.  # noqa: E501
        :rtype: int
        """
        return self._step

    @step.setter
    def step(self, step):
        """Sets the step of this ResponsePlotFile.


        :param step: The step of this ResponsePlotFile.  # noqa: E501
        :type step: int
        """

        self._step = step

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponsePlotFile):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponsePlotFile):
            return True

        return self.to_dict() != other.to_dict()



import os
import zlib
from typing import List

from openapi_client.models import StorageFile
from vessl.util.file_object import UploadableS3Object


class Uploader:
    @classmethod
    def upload(cls, local_path: str, volume_id: int, remote_path: str) -> StorageFile:
        from vessl.volume import create_volume_file, upload_volume_file

        assert os.path.isfile(local_path), f"Invalid path: {local_path}"
        file = create_volume_file(volume_id=volume_id, is_dir=False, path=remote_path)

        mpu = UploadableS3Object(
            local_path,
            file.upload_url.federation_token.bucket,
            file.upload_url.federation_token.key,
            file.upload_url.federation_token.token,
        ).upload()
        return upload_volume_file(volume_id, remote_path)

    @classmethod
    def bulk_upload(
        cls,
        local_base_path: str,
        local_file_paths: List[str],
        volume_id: int,
        remote_base_path: str,
    ) -> List[StorageFile]:
        # TODO: parallel upload

        files = []

        for local_file_path in local_file_paths:
            local_path = os.path.join(local_base_path, local_file_path)
            remote_path = os.path.join(remote_base_path, local_file_path)

            file = cls.upload(local_path, volume_id, remote_path)
            files.append(file)

        return files

    @classmethod
    def calculate_crc32c(filename):
        try:
            with open(filename, "rb") as fh:
                h = 0
                while True:
                    s = fh.read(65536)
                    if not s:
                        break
                    h = zlib.crc32(s, h)
                return "%X" % (h & 0xFFFFFFFF)
        except Exception as e:
            import sentry_sdk

            sentry_sdk.capture_exception(e)
            sentry_sdk.flush()

            return ""

    @classmethod
    def get_paths_in_dir(cls, local_base_path, hashmap=None):
        local_base_path = local_base_path.rstrip("/")

        paths = []
        for root_path, _, file_names in os.walk(local_base_path):
            for file_name in file_names:
                path = os.path.join(root_path, file_name)
                path = (
                    path[len(local_base_path) + 1 :] if path.startswith(local_base_path) else path
                )
                if hashmap and hashmap[path] == cls.calculate_crc32c(
                    os.path.join(local_base_path, path)
                ):
                    continue
                paths.append(path)
        return paths

    @classmethod
    def get_hashmap(cls, local_base_path):
        files = cls.get_paths_in_dir(local_base_path)
        hashmap = dict()
        for file in files:
            path = os.path.join(local_base_path, file)
            hashmap[file] = cls.calculate_crc32c(path)
        return hashmap



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class RunRunExecutionHealthResponse(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'current_status': 'str',
        'last_alive_updated': 'float',
        'registered_ssh_public_keys': 'list[str]'
    }

    attribute_map = {
        'current_status': 'current_status',
        'last_alive_updated': 'last_alive_updated',
        'registered_ssh_public_keys': 'registered_ssh_public_keys'
    }

    def __init__(self, current_status=None, last_alive_updated=None, registered_ssh_public_keys=None, local_vars_configuration=None):  # noqa: E501
        """RunRunExecutionHealthResponse - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._current_status = None
        self._last_alive_updated = None
        self._registered_ssh_public_keys = None
        self.discriminator = None

        self.current_status = current_status
        self.last_alive_updated = last_alive_updated
        self.registered_ssh_public_keys = registered_ssh_public_keys

    @property
    def current_status(self):
        """Gets the current_status of this RunRunExecutionHealthResponse.  # noqa: E501


        :return: The current_status of this RunRunExecutionHealthResponse.  # noqa: E501
        :rtype: str
        """
        return self._current_status

    @current_status.setter
    def current_status(self, current_status):
        """Sets the current_status of this RunRunExecutionHealthResponse.


        :param current_status: The current_status of this RunRunExecutionHealthResponse.  # noqa: E501
        :type current_status: str
        """
        if self.local_vars_configuration.client_side_validation and current_status is None:  # noqa: E501
            raise ValueError("Invalid value for `current_status`, must not be `None`")  # noqa: E501

        self._current_status = current_status

    @property
    def last_alive_updated(self):
        """Gets the last_alive_updated of this RunRunExecutionHealthResponse.  # noqa: E501


        :return: The last_alive_updated of this RunRunExecutionHealthResponse.  # noqa: E501
        :rtype: float
        """
        return self._last_alive_updated

    @last_alive_updated.setter
    def last_alive_updated(self, last_alive_updated):
        """Sets the last_alive_updated of this RunRunExecutionHealthResponse.


        :param last_alive_updated: The last_alive_updated of this RunRunExecutionHealthResponse.  # noqa: E501
        :type last_alive_updated: float
        """
        if self.local_vars_configuration.client_side_validation and last_alive_updated is None:  # noqa: E501
            raise ValueError("Invalid value for `last_alive_updated`, must not be `None`")  # noqa: E501

        self._last_alive_updated = last_alive_updated

    @property
    def registered_ssh_public_keys(self):
        """Gets the registered_ssh_public_keys of this RunRunExecutionHealthResponse.  # noqa: E501


        :return: The registered_ssh_public_keys of this RunRunExecutionHealthResponse.  # noqa: E501
        :rtype: list[str]
        """
        return self._registered_ssh_public_keys

    @registered_ssh_public_keys.setter
    def registered_ssh_public_keys(self, registered_ssh_public_keys):
        """Sets the registered_ssh_public_keys of this RunRunExecutionHealthResponse.


        :param registered_ssh_public_keys: The registered_ssh_public_keys of this RunRunExecutionHealthResponse.  # noqa: E501
        :type registered_ssh_public_keys: list[str]
        """
        if self.local_vars_configuration.client_side_validation and registered_ssh_public_keys is None:  # noqa: E501
            raise ValueError("Invalid value for `registered_ssh_public_keys`, must not be `None`")  # noqa: E501

        self._registered_ssh_public_keys = registered_ssh_public_keys

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RunRunExecutionHealthResponse):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RunRunExecutionHealthResponse):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class RunSpecCreateAPIInput(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'cluster_id': 'int',
        'cluster_node_ids': 'list[int]',
        'description': 'str',
        'docker_credentials_id': 'int',
        'image_url': 'str',
        'kernel_resource_spec_preset_id': 'int',
        'run_spec_input': 'V1RunSpecInputLegacy',
        'title': 'str'
    }

    attribute_map = {
        'cluster_id': 'cluster_id',
        'cluster_node_ids': 'cluster_node_ids',
        'description': 'description',
        'docker_credentials_id': 'docker_credentials_id',
        'image_url': 'image_url',
        'kernel_resource_spec_preset_id': 'kernel_resource_spec_preset_id',
        'run_spec_input': 'run_spec_input',
        'title': 'title'
    }

    def __init__(self, cluster_id=None, cluster_node_ids=None, description=None, docker_credentials_id=None, image_url=None, kernel_resource_spec_preset_id=None, run_spec_input=None, title=None, local_vars_configuration=None):  # noqa: E501
        """RunSpecCreateAPIInput - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._cluster_id = None
        self._cluster_node_ids = None
        self._description = None
        self._docker_credentials_id = None
        self._image_url = None
        self._kernel_resource_spec_preset_id = None
        self._run_spec_input = None
        self._title = None
        self.discriminator = None

        self.cluster_id = cluster_id
        if cluster_node_ids is not None:
            self.cluster_node_ids = cluster_node_ids
        if description is not None:
            self.description = description
        self.docker_credentials_id = docker_credentials_id
        self.image_url = image_url
        self.kernel_resource_spec_preset_id = kernel_resource_spec_preset_id
        self.run_spec_input = run_spec_input
        self.title = title

    @property
    def cluster_id(self):
        """Gets the cluster_id of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The cluster_id of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this RunSpecCreateAPIInput.


        :param cluster_id: The cluster_id of this RunSpecCreateAPIInput.  # noqa: E501
        :type cluster_id: int
        """
        if self.local_vars_configuration.client_side_validation and cluster_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cluster_id`, must not be `None`")  # noqa: E501

        self._cluster_id = cluster_id

    @property
    def cluster_node_ids(self):
        """Gets the cluster_node_ids of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The cluster_node_ids of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: list[int]
        """
        return self._cluster_node_ids

    @cluster_node_ids.setter
    def cluster_node_ids(self, cluster_node_ids):
        """Sets the cluster_node_ids of this RunSpecCreateAPIInput.


        :param cluster_node_ids: The cluster_node_ids of this RunSpecCreateAPIInput.  # noqa: E501
        :type cluster_node_ids: list[int]
        """

        self._cluster_node_ids = cluster_node_ids

    @property
    def description(self):
        """Gets the description of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The description of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this RunSpecCreateAPIInput.


        :param description: The description of this RunSpecCreateAPIInput.  # noqa: E501
        :type description: str
        """

        self._description = description

    @property
    def docker_credentials_id(self):
        """Gets the docker_credentials_id of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The docker_credentials_id of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._docker_credentials_id

    @docker_credentials_id.setter
    def docker_credentials_id(self, docker_credentials_id):
        """Sets the docker_credentials_id of this RunSpecCreateAPIInput.


        :param docker_credentials_id: The docker_credentials_id of this RunSpecCreateAPIInput.  # noqa: E501
        :type docker_credentials_id: int
        """

        self._docker_credentials_id = docker_credentials_id

    @property
    def image_url(self):
        """Gets the image_url of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The image_url of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._image_url

    @image_url.setter
    def image_url(self, image_url):
        """Sets the image_url of this RunSpecCreateAPIInput.


        :param image_url: The image_url of this RunSpecCreateAPIInput.  # noqa: E501
        :type image_url: str
        """
        if self.local_vars_configuration.client_side_validation and image_url is None:  # noqa: E501
            raise ValueError("Invalid value for `image_url`, must not be `None`")  # noqa: E501

        self._image_url = image_url

    @property
    def kernel_resource_spec_preset_id(self):
        """Gets the kernel_resource_spec_preset_id of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The kernel_resource_spec_preset_id of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: int
        """
        return self._kernel_resource_spec_preset_id

    @kernel_resource_spec_preset_id.setter
    def kernel_resource_spec_preset_id(self, kernel_resource_spec_preset_id):
        """Sets the kernel_resource_spec_preset_id of this RunSpecCreateAPIInput.


        :param kernel_resource_spec_preset_id: The kernel_resource_spec_preset_id of this RunSpecCreateAPIInput.  # noqa: E501
        :type kernel_resource_spec_preset_id: int
        """

        self._kernel_resource_spec_preset_id = kernel_resource_spec_preset_id

    @property
    def run_spec_input(self):
        """Gets the run_spec_input of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The run_spec_input of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: V1RunSpecInputLegacy
        """
        return self._run_spec_input

    @run_spec_input.setter
    def run_spec_input(self, run_spec_input):
        """Sets the run_spec_input of this RunSpecCreateAPIInput.


        :param run_spec_input: The run_spec_input of this RunSpecCreateAPIInput.  # noqa: E501
        :type run_spec_input: V1RunSpecInputLegacy
        """
        if self.local_vars_configuration.client_side_validation and run_spec_input is None:  # noqa: E501
            raise ValueError("Invalid value for `run_spec_input`, must not be `None`")  # noqa: E501

        self._run_spec_input = run_spec_input

    @property
    def title(self):
        """Gets the title of this RunSpecCreateAPIInput.  # noqa: E501


        :return: The title of this RunSpecCreateAPIInput.  # noqa: E501
        :rtype: str
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this RunSpecCreateAPIInput.


        :param title: The title of this RunSpecCreateAPIInput.  # noqa: E501
        :type title: str
        """
        if self.local_vars_configuration.client_side_validation and title is None:  # noqa: E501
            raise ValueError("Invalid value for `title`, must not be `None`")  # noqa: E501

        self._title = title

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, RunSpecCreateAPIInput):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, RunSpecCreateAPIInput):
            return True

        return self.to_dict() != other.to_dict()



import json
from http import HTTPStatus
from typing import Dict, List

import requests

from openapi_client.models import (
    ResponsePipeline,
    ResponsePipelineExecution,
    ResponsePipelineStepExecution,
)
from openapi_client.models.pipeline_step_execution_variable_update_api_input import (
    PipelineStepExecutionVariableUpdateAPIInput,
)
from vessl import vessl_api
from vessl.organization import _get_organization_name
from vessl.project import _get_project_name
from vessl.util.constant import (
    GET_ARGUMENT_VALUE_URL,
    GET_CONTEXT_VARIABLE_URL,
    UPDATE_CONTEXT_VARIABLE_URL,
)
from vessl.util.exception import BadRequestError, NotFoundError, VesslException


class StepContextVariable(object):
    def __init__(self, step: str, key: str, value: str):
        self.step_name = step
        self.key = key
        self.value = value

    def to_json(self):
        return json.dumps(self, default=lambda o: o.__dict__, indent=4)


def read_pipeline(pipeline_name: str, **kwargs) -> ResponsePipeline:
    return vessl_api.pipeline_read_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        pipeline_name=pipeline_name,
    ).results


def read_pipeline_execution(pipeline_name: str, number: int, **kwargs) -> ResponsePipelineExecution:
    return vessl_api.pipeline_execution_read_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        pipeline_name=pipeline_name,
        number=number,
    ).results


def list_pipeline(**kwargs) -> List[ResponsePipeline]:
    return vessl_api.pipeline_list_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
    ).results


def list_pipeline_execution(pipeline_name: str, **kwargs) -> List[ResponsePipelineExecution]:
    return vessl_api.pipeline_execution_list_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        pipeline_name=pipeline_name,
    ).results


def update_pipeline_step_execution_variable(
    pipeline_name: str, number: int, step_execution_id: int, variables: Dict[str, str], **kwargs
) -> ResponsePipelineStepExecution:
    return vessl_api.pipeline_step_execution_variable_update_api(
        organization_name=_get_organization_name(**kwargs),
        project_name=_get_project_name(**kwargs),
        pipeline_name=pipeline_name,
        number=number,
        step_exec_id=step_execution_id,
        pipeline_step_execution_variable_update_api_input=PipelineStepExecutionVariableUpdateAPIInput(
            variables=variables
        ),
    )


def update_context_variables(data: Dict[str, str], **kwargs):
    # TODO: Works only in pipeline step execution contexts.
    # TODO: Find current running pipelines.
    header = {"Context-Type": "application/json"}
    resp = requests.post(
        UPDATE_CONTEXT_VARIABLE_URL,
        headers=header,
        data=json.dumps(data),
    )
    if resp.status_code != HTTPStatus.CREATED and resp.status_code != HTTPStatus.OK:
        if resp.status_code == HTTPStatus.BAD_REQUEST:
            raise BadRequestError(f"Update context variables failed. \n data: {data}")
        else:
            raise VesslException()


def get_argument_value(key: str, **kwargs) -> StepContextVariable:
    # TODO: works only in pipeline step execution contexts.
    resp = requests.get(GET_ARGUMENT_VALUE_URL + f"/{key}")

    if resp.status_code != HTTPStatus.OK:
        if resp.status_code == HTTPStatus.BAD_REQUEST:
            raise BadRequestError(f"Get argument value failed. \n key: {key}")
        elif resp.status_code == HTTPStatus.NOT_FOUND:
            raise NotFoundError(f"Argument not found. \n key: {key}")
        else:
            raise VesslException()

    parsed_json = json.loads(resp.content)
    # @XXX(seokju) skip json parse temporarily
    return parsed_json["value"]
    # argument_value = StepContextVariable(**parsed_json)
    # return argument_value


def get_context_variable(step_name: str, key: str, **kwargs) -> StepContextVariable:
    # TODO: works only in pipeline step execution contexts.
    resp = requests.get(GET_CONTEXT_VARIABLE_URL + f"/{step_name}/{key}")

    if resp.status_code != HTTPStatus.OK:
        if resp.status_code == HTTPStatus.BAD_REQUEST:
            raise BadRequestError(
                f"Get context variable failed. \n step_name: {step_name}, key: {key}"
            )
        elif resp.status_code == HTTPStatus.NOT_FOUND:
            raise NotFoundError(
                f"Context variable not found. \n step_name: {step_name}, key: {key}"
            )
        else:
            raise VesslException()

    parsed_json = json.loads(resp.content)
    # @XXX(seokju) skip json parse temporarily
    return parsed_json["value"]
    # context_variable = StepContextVariable(**parsed_json)
    # return context_variable


def get_context_variables(
    pipeline_name: str, step_name: str, **kwargs
) -> List[StepContextVariable]:
    raise NotImplementedError



import os
import tempfile
from pathlib import Path

import humanfriendly
from dotenv import dotenv_values

envvar = {**os.environ, **dotenv_values(".env")}

VESSL_ENV = envvar.get(
    "VESSL_ENV", "prod"
)  # default prod because cli and sdk should be packaged without .env file


# should be called at the end of this file as other values could be set across thie file, see below
# you should add any new envvar related constants here
def _print_envvar_related_constants_on_dev():
    if VESSL_ENV != "prod":
        print("Running in dev mode.. showing envvar related constants")
        print(
            f"""
>> VESSL_ENV: {VESSL_ENV}
>> WEB_HOST: {WEB_HOST}
>> API_HOST: {API_HOST}
>> SENTRY_DSN: {SENTRY_DSN}
>> DEFAULT_VESSL_SIDECAR_HOST: {DEFAULT_VESSL_SIDECAR_HOST}
>> PARALLEL_WORKERS: {PARALLEL_WORKERS}
"""
        )


VESSL_LOG_LEVEL_DEBUG = "DEBUG"
VESSL_LOG_LEVEL_INFO = "INFO"
VESSL_LOG_LEVEL_WARNING = "WARNING"
VESSL_LOG_LEVEL_ERROR = "ERROR"
VESSL_LOG_LEVEL_LEVELS = [
    VESSL_LOG_LEVEL_DEBUG,
    VESSL_LOG_LEVEL_INFO,
    VESSL_LOG_LEVEL_WARNING,
    VESSL_LOG_LEVEL_ERROR,
]

VESSL_LOG_LEVEL = (
    envvar.get("VESSL_LOG")
    if envvar.get("VESSL_LOG") in VESSL_LOG_LEVEL_LEVELS
    else VESSL_LOG_LEVEL_WARNING
)
DEFAULT_WEB_HOST = "https://vessl.ai" if VESSL_ENV == "prod" else "https://dev.vssl.ai"
WEB_HOST = envvar.get("VESSL_WEB_HOST", DEFAULT_WEB_HOST)
DEFAULT_API_HOST = "https://api.vessl.ai" if VESSL_ENV == "prod" else "https://api.dev.vssl.ai"
API_HOST = envvar.get("VESSL_API_HOST", DEFAULT_API_HOST)

SENTRY_DSN = envvar.get(
    "SENTRY_DSN",
    "https://e46fcd750b3a443fbd5b9dbc970e4ecf@o386227.ingest.sentry.io/5911639",
)
ENABLE_SENTRY = envvar.get("ENABLE_SENTRY", "true") == "true"

VESSL_HELM_REPO = "https://vessl-ai.github.io/helm-charts"
VESSL_HELM_CHART_NAME = "vessl"

K0S_VERSION = "v1.25.12-k0s.0"

DEFAULT_VESSL_SIDECAR_HOST = envvar.get("VESSL_SIDECAR_HOST", "http://localhost:3000")
UPDATE_CONTEXT_VARIABLE_URL = f"{DEFAULT_VESSL_SIDECAR_HOST}/store"
GET_ARGUMENT_VALUE_URL = f"{DEFAULT_VESSL_SIDECAR_HOST}/argument"
GET_CONTEXT_VARIABLE_URL = f"{DEFAULT_VESSL_SIDECAR_HOST}/context"

LOGIN_TIMEOUT_SECONDS = 160
ACCESS_TOKEN_ENV_VAR = "VESSL_ACCESS_TOKEN"
DEFAULT_ORGANIZATION_ENV_VAR = "VESSL_DEFAULT_ORGANIZATION"
DEFAULT_PROJECT_ENV_VAR = "VESSL_DEFAULT_PROJECT"
CREDENTIALS_FILE_ENV_VAR = "VESSL_CREDENTIALS_FILE"
PROJECT_NAME_ENV_VAR = "VESSL_PROJECT_NAME"

CLUSTER_KUBECONFIG_ENV_VAR = "VESSL_CLUSTER_KUBECONFIG"
CLUSTER_MODE_SINGLE = "single"
CLUSTER_MODE_MULTI_NODE = "multi"

PROJECT_TYPE_VERSION_CONTROL = "version-control"
PROJECT_TYPES = [PROJECT_TYPE_VERSION_CONTROL]

DATASET_PATH_SCHEME_S3 = "s3://"
DATASET_PATH_SCHEME_GS = "gs://"

DATASET_VERSION_HASH_LATEST = "latest"

PROCESSOR_TYPE_CPU = "CPU"
PROCESSOR_TYPE_GPU = "GPU"
PROCESSOR_TYPES = [PROCESSOR_TYPE_CPU, PROCESSOR_TYPE_GPU]

SWEEP_OBJECTIVE_TYPE_MAXIMIZE = "maximize"
SWEEP_OBJECTIVE_TYPE_MINIMIZE = "minimize"
SWEEP_OBJECTIVE_TYPES = [SWEEP_OBJECTIVE_TYPE_MAXIMIZE, SWEEP_OBJECTIVE_TYPE_MINIMIZE]

SWEEP_ALGORITHM_TYPE_GRID = "grid"
SWEEP_ALGORITHM_TYPE_RANDOM = "random"
SWEEP_ALGORITHM_TYPE_BAYESIAN = "bayesian"
SWEEP_ALGORITHM_TYPES = [
    SWEEP_ALGORITHM_TYPE_GRID,
    SWEEP_ALGORITHM_TYPE_RANDOM,
    SWEEP_ALGORITHM_TYPE_BAYESIAN,
]

SWEEP_PARAMETER_TYPE_CATEGORICAL = "categorical"
SWEEP_PARAMETER_TYPE_INT = "int"
SWEEP_PARAMETER_TYPE_DOUBLE = "double"
SWEEP_PARAMETER_TYPES = [
    SWEEP_PARAMETER_TYPE_CATEGORICAL,
    SWEEP_PARAMETER_TYPE_INT,
    SWEEP_PARAMETER_TYPE_DOUBLE,
]

SWEEP_PARAMETER_RANGE_TYPE_SPACE = "space"
SWEEP_PARAMETER_RANGE_TYPE_LIST = "list"
SWEEP_PARAMETER_RANGE_TYPES = [
    SWEEP_PARAMETER_RANGE_TYPE_SPACE,
    SWEEP_PARAMETER_RANGE_TYPE_LIST,
]

MODEL_SOURCE_EXPERIMENT = "experiment"
MODEL_SOURCE_LOCAL = "local"

SOURCE_TYPE_CODE = "code"
SOURCE_TYPE_ARCHIVE_FILE = "archive-file"
SOURCE_TYPE_OBJECT_STORAGE = "object-storage"
SOURCE_TYPE_DATASET = "dataset"
SOURCE_TYPE_DATASET_VERSION = "dataset-version"
SOURCE_TYPE_MODEL_VOLUME = "model-volume"
SOURCE_TYPE_EMPTY_DIR = "empty-dir"
SOURCE_TYPE_OUTPUT = "output"
SOURCE_TYPE_PROJECT = "project"
SOURCE_TYPE_OUTPUT = "output"
SOURCE_TYPE_OBJECT_STORAGE = "object-storage"

MOUNT_PATH_EMPTY_DIR = "/root/"
MOUNT_PATH_OUTPUT = "/output/"
MOUNT_PATH_PROJECT = "/root/{}"

EXPERIMENT_WORKING_DIR = "/root/"

FRAMEWORK_TYPE_PYTORCH = "pytorch"
FRAMEWORK_TYPE_TENSORFLOW = "tensorflow"
FRAMEWORK_TYPES = (FRAMEWORK_TYPE_PYTORCH, FRAMEWORK_TYPE_TENSORFLOW)

PARALLEL_WORKERS = envvar.get("VESSL_PARALLEL_WORKERS", 20)

VESSL_MEDIA_PATH = "vessl-media"
VESSL_IMAGE_PATH = "images"
VESSL_AUDIO_PATH = "audio"
VESSL_PLOTS_FILETYPE_IMAGE = "image"
VESSL_PLOTS_FILETYPE_IMAGES = "images"
VESSL_PLOTS_FILETYPE_AUDIO = "audio"

WORKSPACE_BACKUP_MAX_SIZE = 15 * 1024 * 1024 * 1024
WORKSPACE_BACKUP_MAX_SIZE_FORMATTED = humanfriendly.format_size(
    WORKSPACE_BACKUP_MAX_SIZE, binary=True
)

TEMP_DIR = tempfile.gettempdir()

SSH_CONFIG_PATH = os.path.join(Path.home(), ".ssh", "config")
SSH_CONFIG_FORMAT = """Host {host}
    User root
    Hostname {hostname}
    Port {port}
    StrictHostKeyChecking accept-new
    CheckHostIP no
"""
SSH_PUBLIC_KEY_PATH = os.path.join(Path.home(), ".ssh", "id_ed25519.pub")
SSH_PRIVATE_KEY_PATH = os.path.join(Path.home(), ".ssh", "id_ed25519")


class colors:
    OKCYAN = "\033[96m"
    OKGREEN = "\033[92m"
    WARNING = "\033[93m"
    FAIL = "\033[91m"
    ENDC = "\033[0m"
    GREY = "\033[38;5;244m"


LOGO = f"""{colors.OKCYAN}
        $$\    $$\ $$$$$$$$\  $$$$$$\   $$$$$$\  $$\              $$$$$$\  $$$$$$\ 
        $$ |   $$ |$$  _____|$$  __$$\ $$  __$$\ $$ |            $$  __$$\ \_$$  _|
        $$ |   $$ |$$ |      $$ /  \__|$$ /  \__|$$ |            $$ /  $$ |  $$ |  
        \$$\  $$  |$$$$$\    \$$$$$$\  \$$$$$$\  $$ |            $$$$$$$$ |  $$ |  
         \$$\$$  / $$  __|    \____$$\  \____$$\ $$ |            $$  __$$ |  $$ |  
          \$$$  /  $$ |      $$\   $$ |$$\   $$ |$$ |            $$ |  $$ |  $$ |  
           \$  /   $$$$$$$$\ \$$$$$$  |\$$$$$$  |$$$$$$$$\       $$ |  $$ |$$$$$$\ 
            \_/    \________| \______/  \______/ \________|      \__|  \__|\______|
        {colors.OKCYAN}"""

NANO_GPT = """
name: nanogpt
image: nvcr.io/nvidia/pytorch:22.03-py3
resources:
  cluster: aws-apne2
  preset: v1.v100-1.mem-52
import:
  /root/examples: git://github.com/vessl-ai/examples
export:
  /output: vessl-artifact://
run:
  - workdir: /root/examples/nanogpt
    command: |
      pip install torchaudio -f https://download.pytorch.org/whl/cu111/torch_stable.html
      pip install transformers datasets tiktoken wandb tqdm
      python data/shakespeare_char/prepare.py
      python train.py config/train_shakespeare_char.py
      python sample.py --out_dir=out-shakespeare-char
"""
STABLE_DIFFUSION = """
name : stable-diffusion
resources:
  cluster: aws-apne2
  preset: v1.v100-1.mem-52
image: quay.io/vessl-ai/ngc-pytorch-kernel:22.12-py3-202301160809
run:
  - workdir: /root/examples/stable-diffusion
    command: |
      bash ./setup.sh && streamlit run main.py
import:
  /root/examples: git://github.com/vessl-ai/examples
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
ports:
  - 8501
"""

SEGMENT_ANYTHING = """
name : segment-anything
resources:
  cluster: aws-apne2
  preset: v1.v100-1.mem-52
image: nvcr.io/nvidia/pytorch:21.05-py3
run:
  - workdir: /root/examples/segment-anything
    command: |
      bash ./setup.sh
import:
  /root/examples: git://github.com/vessl-ai/examples
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
ports:
  - 8501
"""

LANGCHAIN = """
name : langchain
resources:
  cluster: aws-apne2
  preset: v1.cpu-4.mem-12
image: quay.io/vessl-ai/kernels:py38-202303150331
run:
  - workdir: /root/examples/langchain/question_answering/
    command: |
      bash ./run.sh
import:
  /root/examples: git://github.com/vessl-ai/examples
interactive:
  max_runtime: 24h
  jupyter:
    idle_timeout: 120m
ports:
  - 8501
"""
THIN_PLATE_SPLINE_MOTION = """
name: Thin-Plate-Spline-Motion-Model
image: nvcr.io/nvidia/pytorch:21.05-py3
resources:
  cluster: aws-apne2
  preset: v1.v100-1.mem-52
run:
  - workdir: /root/examples/thin-plate-spline-motion-model
    command: |
      pip install -r requirements.txt
      pip install vessl
      python run.py --config config/vox-256.yaml --device_ids 0
      cp -r ./logs /output
import:
  /root/examples: git://github.com/vessl-ai/examples
  /root/examples/vox: s3://vessl-public-apne2/vessl_run_datasets/vox/
export
  /output: vessl-artifact://
"""


_print_envvar_related_constants_on_dev()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class ResponsePipelineExecution(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'arguments': 'list[ResponsePipelineVariable]',
        'created_dt': 'datetime',
        'end_dt': 'datetime',
        'number': 'int',
        'pipeline_id': 'int',
        'pipeline_name': 'str',
        'reason': 'str',
        'status': 'str',
        'step_dependencies': 'list[ResponsePipelineStepDependency]',
        'step_executions': 'list[ResponseReducedPipelineStepExecution]',
        'triggered_by': 'ResponseUser'
    }

    attribute_map = {
        'arguments': 'arguments',
        'created_dt': 'created_dt',
        'end_dt': 'end_dt',
        'number': 'number',
        'pipeline_id': 'pipeline_id',
        'pipeline_name': 'pipeline_name',
        'reason': 'reason',
        'status': 'status',
        'step_dependencies': 'step_dependencies',
        'step_executions': 'step_executions',
        'triggered_by': 'triggered_by'
    }

    def __init__(self, arguments=None, created_dt=None, end_dt=None, number=None, pipeline_id=None, pipeline_name=None, reason=None, status=None, step_dependencies=None, step_executions=None, triggered_by=None, local_vars_configuration=None):  # noqa: E501
        """ResponsePipelineExecution - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._arguments = None
        self._created_dt = None
        self._end_dt = None
        self._number = None
        self._pipeline_id = None
        self._pipeline_name = None
        self._reason = None
        self._status = None
        self._step_dependencies = None
        self._step_executions = None
        self._triggered_by = None
        self.discriminator = None

        self.arguments = arguments
        self.created_dt = created_dt
        self.end_dt = end_dt
        self.number = number
        self.pipeline_id = pipeline_id
        self.pipeline_name = pipeline_name
        if reason is not None:
            self.reason = reason
        self.status = status
        self.step_dependencies = step_dependencies
        self.step_executions = step_executions
        if triggered_by is not None:
            self.triggered_by = triggered_by

    @property
    def arguments(self):
        """Gets the arguments of this ResponsePipelineExecution.  # noqa: E501


        :return: The arguments of this ResponsePipelineExecution.  # noqa: E501
        :rtype: list[ResponsePipelineVariable]
        """
        return self._arguments

    @arguments.setter
    def arguments(self, arguments):
        """Sets the arguments of this ResponsePipelineExecution.


        :param arguments: The arguments of this ResponsePipelineExecution.  # noqa: E501
        :type arguments: list[ResponsePipelineVariable]
        """
        if self.local_vars_configuration.client_side_validation and arguments is None:  # noqa: E501
            raise ValueError("Invalid value for `arguments`, must not be `None`")  # noqa: E501

        self._arguments = arguments

    @property
    def created_dt(self):
        """Gets the created_dt of this ResponsePipelineExecution.  # noqa: E501


        :return: The created_dt of this ResponsePipelineExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._created_dt

    @created_dt.setter
    def created_dt(self, created_dt):
        """Sets the created_dt of this ResponsePipelineExecution.


        :param created_dt: The created_dt of this ResponsePipelineExecution.  # noqa: E501
        :type created_dt: datetime
        """
        if self.local_vars_configuration.client_side_validation and created_dt is None:  # noqa: E501
            raise ValueError("Invalid value for `created_dt`, must not be `None`")  # noqa: E501

        self._created_dt = created_dt

    @property
    def end_dt(self):
        """Gets the end_dt of this ResponsePipelineExecution.  # noqa: E501


        :return: The end_dt of this ResponsePipelineExecution.  # noqa: E501
        :rtype: datetime
        """
        return self._end_dt

    @end_dt.setter
    def end_dt(self, end_dt):
        """Sets the end_dt of this ResponsePipelineExecution.


        :param end_dt: The end_dt of this ResponsePipelineExecution.  # noqa: E501
        :type end_dt: datetime
        """

        self._end_dt = end_dt

    @property
    def number(self):
        """Gets the number of this ResponsePipelineExecution.  # noqa: E501


        :return: The number of this ResponsePipelineExecution.  # noqa: E501
        :rtype: int
        """
        return self._number

    @number.setter
    def number(self, number):
        """Sets the number of this ResponsePipelineExecution.


        :param number: The number of this ResponsePipelineExecution.  # noqa: E501
        :type number: int
        """
        if self.local_vars_configuration.client_side_validation and number is None:  # noqa: E501
            raise ValueError("Invalid value for `number`, must not be `None`")  # noqa: E501

        self._number = number

    @property
    def pipeline_id(self):
        """Gets the pipeline_id of this ResponsePipelineExecution.  # noqa: E501


        :return: The pipeline_id of this ResponsePipelineExecution.  # noqa: E501
        :rtype: int
        """
        return self._pipeline_id

    @pipeline_id.setter
    def pipeline_id(self, pipeline_id):
        """Sets the pipeline_id of this ResponsePipelineExecution.


        :param pipeline_id: The pipeline_id of this ResponsePipelineExecution.  # noqa: E501
        :type pipeline_id: int
        """
        if self.local_vars_configuration.client_side_validation and pipeline_id is None:  # noqa: E501
            raise ValueError("Invalid value for `pipeline_id`, must not be `None`")  # noqa: E501

        self._pipeline_id = pipeline_id

    @property
    def pipeline_name(self):
        """Gets the pipeline_name of this ResponsePipelineExecution.  # noqa: E501


        :return: The pipeline_name of this ResponsePipelineExecution.  # noqa: E501
        :rtype: str
        """
        return self._pipeline_name

    @pipeline_name.setter
    def pipeline_name(self, pipeline_name):
        """Sets the pipeline_name of this ResponsePipelineExecution.


        :param pipeline_name: The pipeline_name of this ResponsePipelineExecution.  # noqa: E501
        :type pipeline_name: str
        """
        if self.local_vars_configuration.client_side_validation and pipeline_name is None:  # noqa: E501
            raise ValueError("Invalid value for `pipeline_name`, must not be `None`")  # noqa: E501

        self._pipeline_name = pipeline_name

    @property
    def reason(self):
        """Gets the reason of this ResponsePipelineExecution.  # noqa: E501


        :return: The reason of this ResponsePipelineExecution.  # noqa: E501
        :rtype: str
        """
        return self._reason

    @reason.setter
    def reason(self, reason):
        """Sets the reason of this ResponsePipelineExecution.


        :param reason: The reason of this ResponsePipelineExecution.  # noqa: E501
        :type reason: str
        """

        self._reason = reason

    @property
    def status(self):
        """Gets the status of this ResponsePipelineExecution.  # noqa: E501


        :return: The status of this ResponsePipelineExecution.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this ResponsePipelineExecution.


        :param status: The status of this ResponsePipelineExecution.  # noqa: E501
        :type status: str
        """
        if self.local_vars_configuration.client_side_validation and status is None:  # noqa: E501
            raise ValueError("Invalid value for `status`, must not be `None`")  # noqa: E501

        self._status = status

    @property
    def step_dependencies(self):
        """Gets the step_dependencies of this ResponsePipelineExecution.  # noqa: E501


        :return: The step_dependencies of this ResponsePipelineExecution.  # noqa: E501
        :rtype: list[ResponsePipelineStepDependency]
        """
        return self._step_dependencies

    @step_dependencies.setter
    def step_dependencies(self, step_dependencies):
        """Sets the step_dependencies of this ResponsePipelineExecution.


        :param step_dependencies: The step_dependencies of this ResponsePipelineExecution.  # noqa: E501
        :type step_dependencies: list[ResponsePipelineStepDependency]
        """
        if self.local_vars_configuration.client_side_validation and step_dependencies is None:  # noqa: E501
            raise ValueError("Invalid value for `step_dependencies`, must not be `None`")  # noqa: E501

        self._step_dependencies = step_dependencies

    @property
    def step_executions(self):
        """Gets the step_executions of this ResponsePipelineExecution.  # noqa: E501


        :return: The step_executions of this ResponsePipelineExecution.  # noqa: E501
        :rtype: list[ResponseReducedPipelineStepExecution]
        """
        return self._step_executions

    @step_executions.setter
    def step_executions(self, step_executions):
        """Sets the step_executions of this ResponsePipelineExecution.


        :param step_executions: The step_executions of this ResponsePipelineExecution.  # noqa: E501
        :type step_executions: list[ResponseReducedPipelineStepExecution]
        """
        if self.local_vars_configuration.client_side_validation and step_executions is None:  # noqa: E501
            raise ValueError("Invalid value for `step_executions`, must not be `None`")  # noqa: E501

        self._step_executions = step_executions

    @property
    def triggered_by(self):
        """Gets the triggered_by of this ResponsePipelineExecution.  # noqa: E501


        :return: The triggered_by of this ResponsePipelineExecution.  # noqa: E501
        :rtype: ResponseUser
        """
        return self._triggered_by

    @triggered_by.setter
    def triggered_by(self, triggered_by):
        """Sets the triggered_by of this ResponsePipelineExecution.


        :param triggered_by: The triggered_by of this ResponsePipelineExecution.  # noqa: E501
        :type triggered_by: ResponseUser
        """

        self._triggered_by = triggered_by

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ResponsePipelineExecution):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, ResponsePipelineExecution):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmPageInfoWithCount(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'end_cursor_id': 'int',
        'end_cursor_value': 'str',
        'has_next_page': 'bool',
        'has_previous_page': 'bool',
        'start_cursor_id': 'int',
        'start_cursor_value': 'str',
        'total_count': 'int'
    }

    attribute_map = {
        'end_cursor_id': 'endCursorID',
        'end_cursor_value': 'endCursorValue',
        'has_next_page': 'hasNextPage',
        'has_previous_page': 'hasPreviousPage',
        'start_cursor_id': 'startCursorID',
        'start_cursor_value': 'startCursorValue',
        'total_count': 'total_count'
    }

    def __init__(self, end_cursor_id=None, end_cursor_value=None, has_next_page=None, has_previous_page=None, start_cursor_id=None, start_cursor_value=None, total_count=None, local_vars_configuration=None):  # noqa: E501
        """OrmPageInfoWithCount - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._end_cursor_id = None
        self._end_cursor_value = None
        self._has_next_page = None
        self._has_previous_page = None
        self._start_cursor_id = None
        self._start_cursor_value = None
        self._total_count = None
        self.discriminator = None

        if end_cursor_id is not None:
            self.end_cursor_id = end_cursor_id
        self.end_cursor_value = end_cursor_value
        if has_next_page is not None:
            self.has_next_page = has_next_page
        if has_previous_page is not None:
            self.has_previous_page = has_previous_page
        if start_cursor_id is not None:
            self.start_cursor_id = start_cursor_id
        self.start_cursor_value = start_cursor_value
        self.total_count = total_count

    @property
    def end_cursor_id(self):
        """Gets the end_cursor_id of this OrmPageInfoWithCount.  # noqa: E501


        :return: The end_cursor_id of this OrmPageInfoWithCount.  # noqa: E501
        :rtype: int
        """
        return self._end_cursor_id

    @end_cursor_id.setter
    def end_cursor_id(self, end_cursor_id):
        """Sets the end_cursor_id of this OrmPageInfoWithCount.


        :param end_cursor_id: The end_cursor_id of this OrmPageInfoWithCount.  # noqa: E501
        :type end_cursor_id: int
        """

        self._end_cursor_id = end_cursor_id

    @property
    def end_cursor_value(self):
        """Gets the end_cursor_value of this OrmPageInfoWithCount.  # noqa: E501


        :return: The end_cursor_value of this OrmPageInfoWithCount.  # noqa: E501
        :rtype: str
        """
        return self._end_cursor_value

    @end_cursor_value.setter
    def end_cursor_value(self, end_cursor_value):
        """Sets the end_cursor_value of this OrmPageInfoWithCount.


        :param end_cursor_value: The end_cursor_value of this OrmPageInfoWithCount.  # noqa: E501
        :type end_cursor_value: str
        """

        self._end_cursor_value = end_cursor_value

    @property
    def has_next_page(self):
        """Gets the has_next_page of this OrmPageInfoWithCount.  # noqa: E501


        :return: The has_next_page of this OrmPageInfoWithCount.  # noqa: E501
        :rtype: bool
        """
        return self._has_next_page

    @has_next_page.setter
    def has_next_page(self, has_next_page):
        """Sets the has_next_page of this OrmPageInfoWithCount.


        :param has_next_page: The has_next_page of this OrmPageInfoWithCount.  # noqa: E501
        :type has_next_page: bool
        """

        self._has_next_page = has_next_page

    @property
    def has_previous_page(self):
        """Gets the has_previous_page of this OrmPageInfoWithCount.  # noqa: E501


        :return: The has_previous_page of this OrmPageInfoWithCount.  # noqa: E501
        :rtype: bool
        """
        return self._has_previous_page

    @has_previous_page.setter
    def has_previous_page(self, has_previous_page):
        """Sets the has_previous_page of this OrmPageInfoWithCount.


        :param has_previous_page: The has_previous_page of this OrmPageInfoWithCount.  # noqa: E501
        :type has_previous_page: bool
        """

        self._has_previous_page = has_previous_page

    @property
    def start_cursor_id(self):
        """Gets the start_cursor_id of this OrmPageInfoWithCount.  # noqa: E501


        :return: The start_cursor_id of this OrmPageInfoWithCount.  # noqa: E501
        :rtype: int
        """
        return self._start_cursor_id

    @start_cursor_id.setter
    def start_cursor_id(self, start_cursor_id):
        """Sets the start_cursor_id of this OrmPageInfoWithCount.


        :param start_cursor_id: The start_cursor_id of this OrmPageInfoWithCount.  # noqa: E501
        :type start_cursor_id: int
        """

        self._start_cursor_id = start_cursor_id

    @property
    def start_cursor_value(self):
        """Gets the start_cursor_value of this OrmPageInfoWithCount.  # noqa: E501


        :return: The start_cursor_value of this OrmPageInfoWithCount.  # noqa: E501
        :rtype: str
        """
        return self._start_cursor_value

    @start_cursor_value.setter
    def start_cursor_value(self, start_cursor_value):
        """Sets the start_cursor_value of this OrmPageInfoWithCount.


        :param start_cursor_value: The start_cursor_value of this OrmPageInfoWithCount.  # noqa: E501
        :type start_cursor_value: str
        """

        self._start_cursor_value = start_cursor_value

    @property
    def total_count(self):
        """Gets the total_count of this OrmPageInfoWithCount.  # noqa: E501


        :return: The total_count of this OrmPageInfoWithCount.  # noqa: E501
        :rtype: int
        """
        return self._total_count

    @total_count.setter
    def total_count(self, total_count):
        """Sets the total_count of this OrmPageInfoWithCount.


        :param total_count: The total_count of this OrmPageInfoWithCount.  # noqa: E501
        :type total_count: int
        """
        if self.local_vars_configuration.client_side_validation and total_count is None:  # noqa: E501
            raise ValueError("Invalid value for `total_count`, must not be `None`")  # noqa: E501

        self._total_count = total_count

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmPageInfoWithCount):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmPageInfoWithCount):
            return True

        return self.to_dict() != other.to_dict()



import os
import pathlib
from typing import List, Union

from vessl.util.vesslignore_pattern import Pattern


def _parse_file_patterns(filename: str) -> List[Pattern]:
    patterns: List[Pattern] = []

    with open(file=filename, newline="\n") as f:
        for line in f:
            line = line.strip()
            # Ignore empty or comment lines
            if len(line) == 0 or line.startswith("#"):
                continue

            patterns.append(Pattern(line))

    return patterns


def _check_all_matches(
    basename: str, dir_names: List[str], patterns: List[List[Pattern]], is_dir: bool
) -> bool:
    """
    Given a file and list of patterns in parent directories, check if there were
    any matching patterns.
    """
    path = (basename + "/") if is_dir else basename
    while patterns:
        for pattern in patterns[-1]:
            if pattern.matches(path):
                return True
        patterns = patterns[:-1]
        if patterns:
            path = dir_names[-1] + "/" + path
            dir_names = dir_names[:-1]
    return False


def list_files_using_vesslignore(
    scan_root: str, follow_symlinks: bool = True, vesslignore_filename: str = ".vesslignore"
) -> List[str]:
    files: List[str] = []

    def _list_files_recurse(
        target: Union[os.DirEntry, pathlib.Path],
        dir_names: List[str],
        patterns: List[List[Pattern]],
        is_root: bool,
    ):
        """
        Args:
            target:
                Current directory or file.

            dir_names:
                List of names of parent directories.

            patterns:
                List of .vesslignore patterns for each of the parent directories. For directories without
                .vesslignore files, an empty list should still be provided.

            is_root: Whether this is the scan root. Needed to drop the root directory from resulting paths.
        """

        is_dir = (
            target.is_dir(follow_symlinks=follow_symlinks)
            if isinstance(target, os.DirEntry)
            else target.is_dir()
        )
        basename = target.name or "."

        if not is_root:
            if _check_all_matches(basename, dir_names, patterns, is_dir):
                # This file or directory should be ignored. Stop here.
                return
            # Use relative path to scan_root (not the full path that contains scan_root),
            # so that we can use `tar -C`.
            relative_path = os.path.join(*dir_names, basename)
            files.append(relative_path)

        if is_dir:
            # Pick vesslignore entry from sub-entries of this directory.
            sub_entries = list(os.scandir(target))
            vesslignore_entry = next(
                (entry for entry in sub_entries if entry.name == vesslignore_filename), None
            )

            # Add patterns from vesslignore file (if exists).
            new_patterns = []
            if vesslignore_entry is not None and vesslignore_entry.is_file():
                new_patterns = _parse_file_patterns(vesslignore_entry.path)
            patterns = patterns + [new_patterns]

            if not is_root:
                dir_names = dir_names + [basename]

            for sub_entry in sub_entries:
                _list_files_recurse(sub_entry, dir_names, patterns, False)

    _list_files_recurse(pathlib.Path(scan_root), [], [], True)

    return files
